#!/bin/bash


# iterate over the parameters
# if the parameter will be a known key, we will store the next parameter as its value


unset type name is_dhcp is_none address mask gateway ns security ssid password vlanid ifaces state

. $lib/net-mask.sh

mkdir -p "$tmp"

type="$1"
shift
allowed_types=(card wlan alias vlan bridge gsm)
if [[ ${allowed_types[@]} =~ $type ]]; then
    echo "Interface type: $type"
else
    echo "Unknown interface type: $type" >&2
    exit 1
fi

name="$1"
shift

#echo params=$#
while [[ $# -gt 0 ]]; do
    case "$1" in
        dhcp)
            # flag whether the network interface should obtain IP from DHCP
            is_dhcp=1
            echo "IP address: from DHCP"
            ;;
        none)
            # flag whether the network interface should be configured as manual
            is_dhcp=0
            is_none=1
            echo "IP address: none"
            ;;
        address)
            # the static IP address for the interface
            shift;  address="$1"
            echo "IP address: $address"
            ;;
        mask)
            # netmask for the interface
            shift
            mask=$(mask2net "$1") || { echo "Netmask $1 invalid" >&2; exit 1; }
            echo "Netmask: /$mask"
            ;;
        security)
            # wireless lan security type: open, wep, wpa2,...
            shift;  security="$1"
            echo "Wireless security: $security"
            ;;
        ssid)
            # wireless SSID name
            shift;  ssid="$1"
            echo "SSID: $ssid"
            ;;
        password)
            # wireless password
            shift;  password="$1"
            echo "Wireless pasword: $password"
            ;;
        parent)
            # Parent interface for alias or vlan
            shift;  parent="$1"
            echo "Parent interface: $parent"
            ;;
        vlanid)
            # vlanid for vlan
            shift;  vlanid="$1"
            echo "Vlanid: $vlanid"
            ;;
        ifaces)
            # Ports for bridge
            shift; ifaces=""
            while [[ $# -gt 0 ]] && [ "$1" != "---" ]; do
                ifaces="$ifaces $1"
                shift
            done
            echo "Bridge ports: $ifaces"
            ;;
        apn)
            #update the APN
            shift;  apn="$1"
            echo "APN = $apn"
            ;;
        pincode)
            #update PIN code
            shift;  pin="$1"
            echo "PIN = $pin"
            ;;
        phonenumber)
            shift
            phonenumber="$1"
            echo "Phone number = $phonenumber"
            ;;
        enabled)
            # flag whether the network setting should be enabled
            # resulting in file created at /etc/network/interfaces.d/
            state=1
            echo "State: enabled"
            ;;
        disabled)
            # flag whether the network setting should be disabled
            # resulting in file removed from /etc/network/interfaces.d/
            state=0
            echo "State: disabled"
            ;;
        *)
            # the parameter is unknown, skip it
            [ -n "$1" ] && echo "Unknown parameter: $1"
            ;;
    esac
    shift
done

export type name is_dhcp is_none address mask gateway ns security ssid password vlanid ifaces state mac_of_first_port

# check whether interface type is set and is valid
if [ -z "$type" ]; then
    echo "Interface type is required" >&2
    exit 1
fi
# validate the interface name
if [ -z "$name" ]; then
    echo "Interface name is required" >&2
    exit 1
fi
# translate the "default" ethernet name
[ "$name" == "default" ] && name="eth0"
echo "Interface name: $name"

######### Create config for WPA-SUPPLICANT
if [ "${type}" == "wlan" ] && [ "$state" != "0" ] && [ "$ssid" != "" ]; then
    echo "$name" > $tmp/wlan.list
    wpafile="$tmp/wpa_supplicant-$name.conf"
    if [ "$security" == "none" ]; then
        envsubst <$templates/wpa-open.conf > "$wpafile"
    elif [ "$security" == "wep" ]; then
        envsubst <$templates/wpa-wep.conf > "$wpafile"
    else
        envsubst <$templates/wpa-wpa.conf > "$wpafile"
    fi
fi

######### Create or delete BRIDGE  device ##############
if [ "${type}" == "bridge" ]; then
    if [ "$state" != "0" ]; then
        for i in $ifaces; do
            if [ -r "/sys/class/net/$i/address" ]; then
                export mac_of_first_port="$(cat /sys/class/net/$i/address)"
                break
            fi
        done
        envsubst <$templates/netdev.netdev > "$tmp/20-$name.netdev"
        for i in $ifaces; do
            mkdir -p $tmp/20-$i.network.d
            envsubst <$templates/bridgeport.conf > "$tmp/20-$i.network.d/bridgeport-$name.conf"
        done
    else
        exit 0
    fi
fi

######### Create or delete VLAN  device ##############
if [ "${type}" == "vlan" ]; then
    if [ "$state" != "0" ]; then
        if [ -n "$parent" ] && [ -n "$vlanid" ]; then
            envsubst <$templates/netdev.netdev > "$tmp/20-$name.netdev"
            envsubst <$templates/vlanid.conf >> "$tmp/20-$name.netdev"
            mkdir -p $tmp/20-$parent.network.d
            envsubst <$templates/vlanif.conf > "$tmp/20-$parent.network.d/vlanif-$vlanid.conf"
        fi
    else
        exit 0
    fi
fi

######### configure WWAN device ##############
if [ "${type}" == "gsm" ]; then
    . "$lib/net-gsm.sh"
    exit 0
fi

######### Add alias to parent ##############
if [ "${type}" == "alias" ]; then
    if [ "$state" != "0" ]; then
        if [ -n "$parent" ] && [ -n "$address" ]; then # ToDo checking
            mkdir -p $tmp/20-$parent.network.d
            envsubst <$templates/alias.conf >> "$tmp/20-$parent.network.d/alias.conf"
            . "$lib/net-mask.sh"
            net=$(cidr_to_net $address $mask)
            echo "$net" "$mask" "$parent" >> "$tmp/to_check_route.list"
        fi
    fi
    exit 0
fi

######### configure network section ##############
src_file="/lib/systemd/network/$name.template"
dst_file="$tmp/20-$name.network"
if [ -f "$src_file" ]; then
    # Export [Match] section from .network file to tmp
    awk '/^\[Match\]/ {p=1; print;next;} /^\[/ {if (p) {nextfile;}} {if (p) {print;}}' "${src_file}" > "${dst_file}"
else
    # create new [Match] section
    envsubst <$templates/match.network > "${dst_file}"
fi

if [ "$state" == "0" ]; then
    # append always-down status if state is disabled
    envsubst <$templates/disable.conf >> "${dst_file}"
fi
#generate "iface" line based on the mode static/dhcp
if [ "$is_dhcp" = "1" ]; then
    envsubst <$templates/dhcpv4.conf >> "${dst_file}"
    echo "Interface will obtain IP from DHCP"
elif [ "$is_none" = "1" ]; then
    envsubst <$templates/noaddress.conf >> "${dst_file}"
    echo "IP configuration of this interface will be manual"
else
    if [ "$state" != "0" ]; then
        . "$lib/net-mask.sh"
        net=$(cidr_to_net $address $mask)
        echo "$net" "$mask" "$name" >> "$tmp/to_check_route.list"
        envsubst <$templates/address.conf >> "${dst_file}"
    fi
fi
